<?php

namespace FiloBlu\OrderFlowRectifier\Helper;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Quote\Model\QuoteRepository;
use Magento\Quote\Model\Quote\Item\Repository;

class Data
{
    const CHECK_QUOTE_ITEM_ENABLED_CONFIG_PATH = 'order_flow_rectifier/quote_item/quote_item_enable';
    const EMAIL_LIST_CONFIG_PATH = 'order_flow_rectifier/quote_item/email_notification';
    const DELETE_QUOTE_ITEM_CONFIG_PATH = 'order_flow_rectifier/quote_item/delete_quote_item';
    const XML_PATH_STORE_NAME = 'general/store_information/name';
    const PRODUCT_TYPE_CONFIGURABLE = 'configurable';

    protected $config;
    protected $quoteRepository;
    protected $emailNotifyHelper;
    protected $quoteItemRepository;

    public function __construct(
        ScopeConfigInterface $config,
        QuoteRepository $quoteRepository,
        EmailNotification $emailNotifyHelper,
        Repository $quoteItemRepository
    ) {
        $this->config = $config;
        $this->quoteRepository = $quoteRepository;
        $this->emailNotifyHelper = $emailNotifyHelper;
        $this->quoteItemRepository = $quoteItemRepository;
    }

    public function checkQuoteItem($quoteItem, $quoteId)
    {
        $missingQuoteChildren = array();
        $itemData = array();

        foreach ($quoteItem as $key => $item)
        {
            if($item->getProductType() == self::PRODUCT_TYPE_CONFIGURABLE)
            {
                $itemHasChildren = false;
                $itemId = $item->getItemId();
                $quoteItemChildren = $item->getChildren();
                if($quoteItemChildren && !empty($quoteItemChildren))
                {
                    foreach ($quoteItemChildren as $children)
                    {
                        if ($children->getParentItemId() == $itemId)
                        {
                            $itemHasChildren = true;
                        }
                    }
                }

                if (!$itemHasChildren)
                {
                    $productName = (!empty($item->getName()) && !is_null($item->getName())) ? $item->getName() : '';
                    $itemData[] = array(
                        'item_id' => $itemId,
                        'sku' => $item->getSku(),
                        'product_name' => $productName
                    );
                }
            }
        }

        if(!empty($itemData))
        {
            $missingQuoteChildren['quote_id'] = $quoteId;
            $missingQuoteChildren['item_data'] = $itemData;
        }
        
        return $missingQuoteChildren;
    }

    public function getQuoteItem($quoteId)
    {
        $quote = $this->quoteRepository->get($quoteId);
        return $quote->getItems();
    }

    public function checkQuoteItemIsEnabled()
    {
        return $this->config->isSetFlag(self::CHECK_QUOTE_ITEM_ENABLED_CONFIG_PATH);
    }

    public function deleteQuoteItemIsEnabled()
    {
        return $this->config->isSetFlag(self::DELETE_QUOTE_ITEM_CONFIG_PATH);
    }

    public function deleteQuoteItem($quoteId, $itemId)
    {
        $this->quoteItemRepository->deleteById($quoteId, $itemId);
    }

    public function log($toLog)
    {
        $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/order-flow-rectifier-check-quote-item.log');
        $logger = new \Zend\Log\Logger();
        $logger->addWriter($writer);
        $logger->info($toLog);
    }

    public function sendNotificationMail($missingData, $action = '')
    {
        $emailAddressString = $this->config->getValue(self::EMAIL_LIST_CONFIG_PATH);

        if (!is_null($emailAddressString) && $emailAddressString && $emailAddressString !== '') {
            $emailAddress = explode(',', $emailAddressString);
            $emailAddress = array_map('trim', $emailAddress);

            $storeName = $this->config->getValue(self::XML_PATH_STORE_NAME);

            $notificationString = $action != '' ? 'Missing items found during: '.$action : '';
            $notificationString.= '<ul>';
            foreach ($missingData['item_data'] as $val)
            {
                $notificationString.= '<li>missing simple for item_id -> '.$val['item_id'].' (SKU '.$val['sku'].' - product name '.$val['product_name'].')</li>';
            }
            $notificationString.= '</ul>';

            // Datas for email
            $data = [
                'mail_subject' => 'Quote item missing for ' . $storeName,
                'quote' => $missingData['quote_id'],
                'errors' => $notificationString
            ];

            $this->emailNotifyHelper->notify($emailAddress, $data);
        }
    }
}